import { Model, ModelEvent } from "./Model";
import { MergeStrategy } from "./merge/MergeStrategy";
import { CacheStrategy } from "./cache/CacheStrategy";
import { Class } from "mvx-utils";
export declare abstract class DetailModelItem {
    toObject(): object;
    newItem<T extends DetailModelItem>(o: object): T;
    deepCopy(from: DetailModelItem): this;
    shallowCopy(other: DetailModelItem): void;
    clear(): void;
    isInvalid(): boolean;
    abstract getId(): string | null;
}
export declare class DetailModelEvent extends ModelEvent {
    ids: string[];
    constructor(ids: string[]);
}
export declare abstract class DetailModel extends Model {
    set(items: DetailModelItem[]): Promise<void>;
    onlyClearCache(): void;
    setNoPost(items: DetailModelItem[]): Promise<void>;
    setWithStrategy(items: DetailModelItem[], merge: MergeStrategy, post: boolean): Promise<void>;
    protected setImpl<T extends DetailModelItem>(items: T[], merge: MergeStrategy, post: boolean): Promise<void>;
    protected setImplNotSafe<T extends DetailModelItem>(items: T[], merge: MergeStrategy, post: boolean): Promise<void>;
    get<T extends DetailModelItem>(ids: string[], clazz: Class<T>): Promise<Map<string, Readonly<T>>>;
    protected abstract saveToDB(items: DetailModelItem[]): Promise<void>;
    protected abstract loadFromDB<T extends DetailModelItem>(ids: string[], clazz: Class<T>): Promise<Map<string, T>>;
    protected abstract loadFromDBForMerge<T extends DetailModelItem>(id: string): Promise<T | null>;
    protected setCacheStrategy(cache: CacheStrategy<string>): void;
    postEvent(ids: string[]): Promise<void>;
    protected abstract getSubEvent(ids: string[]): DetailModelEvent;
    private details_;
    private cacheStrategy_;
    private runner_;
}
