import { Net } from "./Net";
import { DefaultConstructor, Duration } from "mvx-utils";
import { Method } from "./http/HttpBuilder";
export declare enum Code {
    Success = 200,
    TokenExpire = 401,
    ElseError = 500
}
export declare class CodeError extends Error {
    code: Code;
    constructor(err: string | Error, code?: Code);
}
export interface Decoder {
    decode<T extends object | string>(uri: string, resType: DefaultConstructor<T>, res: string, err: CodeError): [T, CodeError | null];
}
export interface Encoder {
    encode<T extends object>(request: object): [string, Error | null];
}
interface opt {
    encoder: Encoder;
    header: Map<string, string>;
    setToken: (opt: {
        header: Map<string, string>;
        request: object;
    }, token: string) => void;
    method: Method;
    cacheTTL: (codeError: Code) => Duration;
    cacheAble: boolean;
    decoder: Decoder;
}
export declare type HttpOption = (o: opt) => Error | null;
export declare function WithHeader(header: Map<string, string>): HttpOption;
export declare function WithBearerToken(): HttpOption;
export declare function WithBodyToken(tokenKey?: string): HttpOption;
export declare function WithoutToken(): HttpOption;
export declare function WithReqId(): HttpOption;
export declare function WithBody(encoder: Encoder): HttpOption;
export declare function WithFormUrlEncodedBody(): HttpOption;
export declare function WithJsonBody(): HttpOption;
export declare function WithHttpMethod(method: Method): HttpOption;
export declare function WithCache(cacheAble: boolean, cacheTTL?: (code: Code) => Duration): HttpOption;
export declare function WithHttpDecoder(decoder: Decoder): HttpOption;
export declare function SendNoToken<T extends object | string>(uri: string, request: object, resType: DefaultConstructor<T>, net: Net, ...opts: HttpOption[]): Promise<[T, CodeError | null]>;
export declare function Send<T extends object | string>(uri: string, request: object, resType: DefaultConstructor<T>, net: Net, ...opts: HttpOption[]): Promise<[T, CodeError | null]>;
export {};
