var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { Model, ModelEvent } from "./Model";
import { JoinMergeStrategy } from "./merge/JoinMergeStrategy";
import { NoEliminationCacheStrategy } from "./cache/NoEliminationCacheStrategy";
import { Json, nullize } from "mvx-utils";
import { SequentiallyRun } from "./SequentiallyRun";
export class DetailModelItem {
    // 下面两个函数，不同的item可以定制，可以认为一个是序列化为object 一个是从object反序列化
    toObject() {
        return this;
    }
    // 通过o新建一个T，并不会改变this的任何值
    newItem(o) {
        return o;
    }
    deepCopy(from) {
        const [json, err] = new Json().toJson(from);
        if (err) {
            throw err;
        }
        new Json().parseJsonObjectToClass(this, JSON.parse(json));
        return this;
    }
    shallowCopy(other) {
        for (let key in other) {
            if (this.hasOwnProperty(key)) {
                this[key] = other[key];
            }
        }
    }
    clear() {
        nullize(this);
    }
    isInvalid() {
        return this.getId() === null || this.getId() === "";
    }
}
export class DetailModelEvent extends ModelEvent {
    constructor(ids) {
        super();
        this.ids = ids;
    }
}
export class DetailModel extends Model {
    constructor() {
        super(...arguments);
        this.details_ = new Map();
        this.cacheStrategy_ = new NoEliminationCacheStrategy();
        this.runner_ = new SequentiallyRun();
    }
    set(items) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.setImpl(items, new JoinMergeStrategy(), true);
        });
    }
    onlyClearCache() {
        this.details_.clear();
        this.cacheStrategy_.clear();
    }
    setNoPost(items) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.setImpl(items, new JoinMergeStrategy(), false);
        });
    }
    setWithStrategy(items, merge, post) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.setImpl(items, merge, post);
        });
    }
    setImpl(items, merge, post) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise(resolve => {
                this.runner_.Push(() => __awaiter(this, void 0, void 0, function* () {
                    yield this.setImplNotSafe(items, merge, post);
                    resolve();
                }));
            });
        });
    }
    // 因为是异步执行，可能存在此接口被同一个模型同时调用的情况，
    // 在合并的过程中，就可能不是使用的最新的本地数据，会出现数据覆盖的情况，
    // 所以，此函数只能单次的按序调用。
    // 与多线程有点类似，都可能对数据造成错乱，但是异步仍然是在一个线程执行的，
    // 所以不存在关键区代码保护的问题。
    setImplNotSafe(items, merge, post) {
        return __awaiter(this, void 0, void 0, function* () {
            if (items === null || items.length === 0) {
                return;
            }
            let promises = [];
            // 下面这段代码的使用需要后续再考虑，在上次改版时，留下了这段代码，但其实下面的逻辑并没有用到
            // let loadOldFromDb_ = async (id:string, value?: T):
            //   Promise<T|null> => {
            //   if (value !== undefined) {
            //     return value;
            //   }
            //
            //   return (await this.loadFromDBForMerge(id));
            // };
            // find old
            for (let item of items) {
                if (item.isInvalid()) {
                    continue;
                }
                let id = item.getId();
                let v = this.details_.get(id);
                if (v) {
                    promises.push(Promise.resolve(v));
                    continue;
                }
                promises.push(this.loadFromDBForMerge(id));
            }
            let olds = new Map();
            for (let old of yield Promise.all(promises)) {
                if (old !== null && !old.isInvalid()) {
                    olds.set(old.getId(), old);
                }
            }
            // merge
            let newItems = [];
            let changed = false;
            for (let item of items) {
                if (item.isInvalid()) {
                    continue;
                }
                let old = olds.get(item.getId());
                let oldO = null;
                if (old) {
                    oldO = old.toObject();
                }
                let [ret, c] = merge.mergeToOld(oldO, item.toObject());
                newItems.push(item.newItem(ret));
                changed = changed || c;
            }
            yield this.saveToDB(newItems);
            // cache adjust
            let ids = [];
            for (let item of newItems) {
                ids.push(item.getId());
            }
            let eimi = this.cacheStrategy_.adjustCache(ids);
            let eimiSet = new Set(eimi);
            for (let id of eimi) {
                this.details_.delete(id);
            }
            for (let item of newItems) {
                if (eimiSet.has(item.getId())) {
                    continue;
                }
                this.details_.set(item.getId(), item);
            }
            if (changed && post) {
                yield this.postEvent(ids);
            }
        });
    }
    get(ids, clazz) {
        return __awaiter(this, void 0, void 0, function* () {
            let ret = new Map();
            let db = [];
            for (let id of ids) {
                let value = this.details_.get(id);
                if (value === undefined) {
                    db.push(id);
                }
                else {
                    ret.set(id, value);
                }
            }
            if (db.length === 0) {
                return ret;
            }
            let items = yield this.loadFromDB(db, clazz);
            for (let entry of items) {
                ret.set(entry[0], entry[1]);
            }
            let eimi = this.cacheStrategy_.adjustCache([...items.keys()]);
            let eimiSet = new Set(eimi);
            for (let id of eimi) {
                this.details_.delete(id);
            }
            for (let entry of items) {
                if (eimiSet.has(entry[0])) {
                    continue;
                }
                this.details_.set(entry[0], entry[1]);
            }
            return ret;
        });
    }
    // ------------------------
    setCacheStrategy(cache) {
        this.cacheStrategy_ = cache;
    }
    postEvent(ids) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.getPlugin().nc().post(this.getSubEvent(ids));
        });
    }
}
