var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { Model, ModelEvent } from "./Model";
import { Table } from "../db/Table";
import { NullTable } from "../db/NullTable";
// 库的重新初始化，基础模块需要监听此事件，作初始化操作,
// 此事件不应该做业务调用或者调用其他初始模块，可能被调用者还没初始化成功
export class LibraryShouldReInitByUidChanged extends ModelEvent {
}
// 定义 AppInvalidEvent 与 AppReValidEvent 供上层业务使用，
// 这两个事件往往与Me的对应操作相关联，所以定义在此文件中，
// 执行了 InValidate 常常需要发 AppInvalidEvent 事件，但是并不在 InValidate 操作中发出，
// 在实际发通知的地方，除了 Me.InValidate 外，还有其他操作执行后才能发通知
// AppReValidEvent 与 ReValidateWithUid 类似
export class AppInvalidEvent extends ModelEvent {
}
export class AppReValidEvent extends ModelEvent {
}
export class Me extends Model {
    constructor() {
        super(...arguments);
        this.uid_ = null;
        this.table_ = new NullTable();
    }
    static isValidUid(uid) {
        return uid !== Me.empty;
    }
    InValidate() {
        return __awaiter(this, void 0, void 0, function* () {
            return (yield this.setUid(Me.empty));
        });
    }
    ReValidateWithUid(uid) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!Me.isValidUid(uid)) {
                return Error("uid(" + uid + ") is invalid.");
            }
            return (yield this.setUid(uid));
        });
    }
    setUid(uid) {
        return __awaiter(this, void 0, void 0, function* () {
            if (uid === this.uid_) {
                return null;
            }
            this.uid_ = uid;
            let err = yield this.table_.set(Me.uidStr, this.uid_);
            if (err != null) {
                return err;
            }
            yield this.getPlugin().nc().post(new LibraryShouldReInitByUidChanged());
            return null;
        });
    }
    uid() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.uid_ === null) {
                let [ret, err] = yield this.table_.get(Me.uidStr);
                if (err !== null) {
                    ret = Me.empty;
                }
                this.uid_ = ret;
            }
            return this.uid_;
        });
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            this.table_ = new Table(this.getDBM().getShareDBAtSessionStorage(), "Me");
        });
    }
    needClearWhenUidChanged() {
        return false;
    }
}
Me.uidStr = "uid";
Me.empty = "";
