var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { Json, Millisecond } from "mvx-utils";
import { DetailModel } from "./DetailModel";
import { Table } from "../db/Table";
import { NullTable } from "../db/NullTable";
export class TheDetailModel extends DetailModel {
    constructor() {
        super(...arguments);
        this.table_ = new NullTable();
    }
    load(id, clazz) {
        return __awaiter(this, void 0, void 0, function* () {
            let [v, err] = yield this.table_.get(id);
            if (err) {
                return null;
            }
            let ret;
            [ret, err] = new Json().fromJson(v, clazz);
            if (err) {
                console.error(err);
                return null;
            }
            return ret;
        });
    }
    loadFromDB(ids, clazz) {
        return __awaiter(this, void 0, void 0, function* () {
            let ret = new Map();
            for (let id of ids) {
                let v = yield this.load(id, this.getItemClazz());
                if (!v) {
                    continue;
                }
                ret.set(id, v);
            }
            return ret;
        });
    }
    loadFromDBForMerge(id) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.load(id, this.getItemClazz());
        });
    }
    saveToDB(items) {
        return __awaiter(this, void 0, void 0, function* () {
            for (let item of items) {
                if (item.isInvalid()) {
                    continue;
                }
                let id = item.getId();
                let [json, err] = new Json().toJson(item);
                if (!err) {
                    err = yield this.table_.set(id, json, 10 * Millisecond);
                }
                if (err) {
                    console.error(`save id(${id}) to db`, item, err);
                }
            }
        });
    }
    getAllIds() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.table_.getAllKeys();
        });
    }
    getTable() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Table(yield this.getDBM().getSelfDB(), this.getTableName());
        });
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            this.table_ = yield this.getTable();
        });
    }
}
