var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
export class NotificationCenter {
    /**
     * @param plugin
     *
     */
    constructor(plugin) {
        this.allEvent = new Set();
        // 防止正在post时，事件响应中调用 unRegister 等方式造成map在循环中执行delete操作 产生不可预测的异常
        this.postingMap = null;
        this.deleteObservers = new Set();
        this.plugin_ = plugin;
        this.eventId_ = Symbol("eventid");
    }
    getPlugin() {
        return this.plugin_;
    }
    post(event) {
        return __awaiter(this, void 0, void 0, function* () {
            let map = event[this.eventId_] || new Map();
            this.postingMap = map;
            let exe = [];
            for (let [key, value] of map) {
                if (this.deleteObservers.has(key)) {
                    continue;
                }
                exe.push(value(event));
            }
            // 并发执行
            yield Promise.all(exe);
            // delete 需要删除的 observers
            for (let o of this.deleteObservers) {
                map.delete(o);
            }
            if (map.size == 0) {
                // todo event.constructor 是否等于 Class<T>.prototype
                this.allEvent.delete(event.constructor);
            }
            this.deleteObservers.clear();
            this.postingMap = null;
        });
    }
    // 同一个observer多次注册同一个event，最后的执行函数以最后一次的为准
    register(observer, event, clbk) {
        let map = event.prototype[this.eventId_]
            = event.prototype[this.eventId_] || new Map();
        map.set(observer, clbk);
        this.allEvent.add(event);
    }
    unRegister(observer, event) {
        let map = event.prototype[this.eventId_];
        if (!map) {
            return;
        }
        // 正在post
        if (map == this.postingMap) {
            this.deleteObservers.add(observer);
            return;
        }
        map.delete(observer);
        if (map.size == 0) {
            this.allEvent.delete(event);
        }
    }
    unRegisterAll(observer) {
        let del = [];
        for (let event of this.allEvent) {
            let map = event.prototype[this.eventId_];
            if (!map) {
                return;
            }
            // 正在post
            if (map == this.postingMap) {
                this.deleteObservers.add(observer);
                return;
            }
            map.delete(observer);
            if (map.size == 0) {
                del.push(event);
            }
        }
        for (let event of del) {
            this.allEvent.delete(event);
        }
    }
}
