import { ModelFactory } from "./model/ModelFactory";
import { Net } from "./net/Net";
import { NotificationCenter } from "./nc/NotificationCenter";
import { NullDBManager } from "./db/NullDBManager";
export class Plugin {
    constructor(name) {
        this.nets_ = new Map();
        this.mainNet_ = null;
        this.nc_ = new NotificationCenter(this);
        this.mf_ = new ModelFactory(this);
        this.dbManager_ = new NullDBManager(this);
        this.name_ = name;
    }
    static register(name) {
        let p = Plugin.allPlugins.get(name);
        if (p) {
            throw new Error("plugin error: " + name + " has been registered");
        }
        p = new Plugin(name);
        Plugin.allPlugins.set(name, p);
    }
    static getBy(name) {
        let p = Plugin.allPlugins.get(name);
        if (p === undefined) {
            throw new Error("plugin error: " + name + " has NOT been registered");
        }
        return p;
    }
    getName() {
        return this.name_;
    }
    nc() {
        return this.nc_;
    }
    mf() {
        return this.mf_;
    }
    dbManager() {
        return this.dbManager_;
    }
    setDBManager(dbm) {
        this.dbManager_ = dbm;
    }
    net(baseUrl) {
        let net = this.nets_.get(baseUrl);
        if (net === undefined) {
            net = new Net(this, baseUrl);
            this.nets_.set(baseUrl, net);
        }
        return net;
    }
    getMainNet() {
        if (this.mainNet_ === null) {
            throw new Error("main net has not set!");
        }
        return this.mainNet_;
    }
    setMainNet(net) {
        if (this.mainNet_ === null) {
            this.mainNet_ = net;
            // 主网络必须包含于nets集合中，所以这里无论设置的主网络是否在nets中，直接再加入一次
            this.nets_.set(net.getBaseUrl(), net);
            return;
        }
        if (this.mainNet_ !== net) {
            throw new Error("duplicate main net!");
        }
    }
}
Plugin.allPlugins = new Map();
