import {Storage} from "./Storage";
import {Duration} from "mvx-utils";

export class DB {
  public async get(table:string, key:string):Promise<[string, Error|null]> {
    return (await this.storage_.get(this.getFullTable(table), key));
  }

  public async set(table:string, key:string, value:string, delay:Duration):Promise<Error|null> {
    return (await this.storage_.set(this.getFullTable(table), key, value, delay));
  }

  public async remove(table:string, key:string):Promise<Error|null> {
    return (await this.storage_.remove(this.getFullTable(table), key));
  }

  public async getAllKeys(table:string):Promise<string[]> {
    return (await this.storage_.getAllKeys(this.getFullTable(table)));
  }

  private getFullTable(table:string):string{
    return this.name_ + "." + table;
  }

  constructor(storage:Storage, name:string) {
    this.name_ = name;
    this.storage_ = storage;
  }

  private storage_:Storage;
  private readonly name_:string;
}