var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { DB } from "./DB";
import { LibraryShouldReInitByUidChanged, Me } from "../model/Me";
import { NullDB } from "./NullDB";
export class DBManager {
    /**
     *
     * @param plugin
     *
     * 初始化时，不要初始化SelfDB, 因为 self 依赖me, 而按照整个框架，me 依赖于shareDB ,
     * 此时，任何DB都还不能确定创建好，也不能确定初始化的顺序，所以不能在这里初始化selfDB
     */
    constructor(plugin) {
        this.shareAtLocal_ = null;
        this.shareAtSession_ = null;
        this.selfDB_ = null;
        this.plugin_ = plugin;
        // Me.uid 改变时，应该重新创建selfDB
        plugin.nc().register(this, LibraryShouldReInitByUidChanged, (event) => __awaiter(this, void 0, void 0, function* () {
            yield this.createSelfDB();
        }));
    }
    getShareDBAtSessionStorage() {
        if (this.shareAtSession_ === null) {
            this.shareAtSession_ = new DB(this.newSessionStorage(), this.getDBName(DBManager.shareName));
        }
        return this.shareAtSession_;
    }
    getShareDBAtLocalStorage() {
        if (this.shareAtLocal_ === null) {
            this.shareAtLocal_ = new DB(this.newLocalStorage(), this.getDBName(DBManager.shareName));
        }
        return this.shareAtLocal_;
    }
    getSelfDB() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.selfDB_ === null) {
                yield this.createSelfDB();
            }
            return this.selfDB_;
        });
    }
    getPlugin() {
        return this.plugin_;
    }
    createSelfDB() {
        return __awaiter(this, void 0, void 0, function* () {
            let me = yield (yield this.plugin_.mf().getModel(Me)).uid();
            if (!Me.isValidUid(me)) {
                this.selfDB_ = new NullDB(this.newSessionStorage());
            }
            else {
                this.selfDB_ = new DB(this.newSessionStorage(), this.getDBName(me));
            }
        });
    }
    getDBName(name) {
        return this.plugin_.getName() + "." + name;
    }
}
DBManager.shareName = "share";
