var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { Model, ModelEvent } from "./Model";
import { OverWriteMergeStrategy } from "./merge/OverWriteMergeStrategy";
import { Table } from "../db/Table";
import { NullTable } from "../db/NullTable";
import { SequentiallyRun } from "./SequentiallyRun";
export class ListModelEvent extends ModelEvent {
    constructor(listKey) {
        super();
        this.listKey = listKey;
    }
}
export class ListModel extends Model {
    constructor() {
        super(...arguments);
        this.list_ = new Map();
        this.table_ = new NullTable();
        this.runner_ = new SequentiallyRun();
    }
    get(key) {
        return __awaiter(this, void 0, void 0, function* () {
            let value = this.list_.get(key);
            if (value !== undefined) {
                return value;
            }
            let [val, err] = yield this.table_.get(key);
            if (err) {
                return null;
            }
            value = JSON.parse(val);
            if (value === null || value === undefined) {
                return null;
            }
            this.list_.set(key, value);
            return value;
        });
    }
    set(key, list) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.setWithStrategy(key, list, new OverWriteMergeStrategy(), true);
        });
    }
    add(key, value) {
        return __awaiter(this, void 0, void 0, function* () {
            let list = yield this.get(key);
            if (list === null) {
                list = [value];
            }
            list = [...list, value];
            yield this.set(key, list);
            return;
        });
    }
    remove(key, value) {
        return __awaiter(this, void 0, void 0, function* () {
            let list = yield this.get(key);
            if (list === null) {
                return;
            }
            list = list.filter((id) => id !== value);
            yield this.set(key, list);
            return;
        });
    }
    setNoPost(key, list) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.setWithStrategy(key, list, new OverWriteMergeStrategy(), false);
        });
    }
    setWithStrategy(key, list, merge, post) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise(resolve => {
                this.runner_.Push(() => __awaiter(this, void 0, void 0, function* () {
                    yield this.setWithStrategyNotSafe(key, list, merge, post);
                    resolve();
                }));
            });
        });
    }
    // 同 DetailModel
    setWithStrategyNotSafe(key, list, merge, post) {
        return __awaiter(this, void 0, void 0, function* () {
            let old = yield this.get(key);
            let changed = false;
            if (old === null) {
                changed = true;
                old = [];
            }
            let ret = merge.mergeToOldArray(old, list);
            changed = changed || ret;
            this.list_.set(key, old);
            yield this.table_.set(key, JSON.stringify(old));
            if (changed && post) {
                yield this.getPlugin().nc().post(new ListModelEvent(key));
            }
        });
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            this.table_ = new Table(yield this.getDBM().getSelfDB(), "List");
        });
    }
}
