var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { LibraryShouldReInitByUidChanged } from "./Me";
export class ModelFactory {
    constructor(plugin) {
        this.models_ = new Map();
        this.plugin_ = plugin;
        this.factory_ = Symbol("factory");
        plugin.nc().register(this, LibraryShouldReInitByUidChanged, (event) => __awaiter(this, void 0, void 0, function* () {
            yield this.clearOnUidChanged();
        }));
    }
    getModel(modelClazz) {
        return __awaiter(this, void 0, void 0, function* () {
            let id = modelClazz[this.factory_];
            if (id === undefined) {
                id = Symbol("modelID");
                modelClazz[this.factory_] = id;
            }
            let model = this.models_.get(id);
            if (model === undefined) {
                model = new modelClazz(this.plugin_);
                yield model.init();
                this.models_.set(id, model);
            }
            return model;
        });
    }
    clearOnUidChanged() {
        return __awaiter(this, void 0, void 0, function* () {
            let willDeleteIDs = [];
            for (let [id, model] of this.models_) {
                if (model.needClearWhenUidChanged()) {
                    willDeleteIDs.push(id);
                }
            }
            for (let id of willDeleteIDs) {
                let model = this.models_.get(id);
                if (model !== undefined) {
                    yield model.willDeInit();
                }
                this.models_.delete(id);
            }
        });
    }
}
