var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { Token } from "../model/Token";
/**
 * 401 表示net层不通，连通net层的方法常常是 login
 */
export class Net {
    constructor(plugin, baseUrl) {
        this.has401_ = false;
        this.creator_ = (headerAPIKey) => {
            throw new Error("Please setHttpBuilderCreator");
        };
        this.mainNet_ = false;
        this.net401Delegate_ = ((net) => { return Promise.resolve(); });
        this.plugin_ = plugin;
        this.baseUrl_ = baseUrl;
    }
    // 连续的多次401，只处理一次
    process401() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.has401_) {
                return;
            }
            this.has401_ = true;
            yield this.net401Delegate_(this);
        });
    }
    set401Delegate(net401Delegate) {
        this.net401Delegate_ = net401Delegate;
    }
    set401() {
        this.has401_ = true;
    }
    is401() {
        return this.has401_;
    }
    // 通常清除401的场景：1、任何需要token的接口，非401返回时；2、登录成功时(给net设置有效的token值)；3、业务其他逻辑主动清除
    clear401() {
        this.has401_ = false;
    }
    getBaseUrl() {
        return this.baseUrl_;
    }
    // 每次获取builder时，在net层都应该调用生成函数生成一个builder 而不能在net层直接缓存一个builder直接返回
    // 对于http等无状态的请求，每次都应该是一个全新的builder 而对于其他类型的协议，有可能不是每次都是全新的一个builder
    // 具体如何生成，应该是creator的责任，而不是net来负责，net负责的是会话级别的连接，builder负责的是具体的协议
    setHttpBuilderCreator(creator) {
        this.creator_ = creator;
    }
    getHttpBuilder() {
        return this.creator_(this.baseUrl_);
    }
    setToken(token) {
        return __awaiter(this, void 0, void 0, function* () {
            if (token !== Token.empty) {
                this.clear401();
            }
            yield (yield this.plugin_.mf().getModel(Token)).set(this.baseUrl_, token);
        });
    }
    getToken() {
        return __awaiter(this, void 0, void 0, function* () {
            return (yield (yield this.plugin_.mf().getModel(Token)).get(this.baseUrl_));
        });
    }
    clearToken() {
        return __awaiter(this, void 0, void 0, function* () {
            yield (yield this.plugin_.mf().getModel(Token)).clear(this.baseUrl_);
        });
    }
    setToMainNet() {
        this.plugin_.setMainNet(this);
        this.mainNet_ = true;
        return this;
    }
    isMainNet() {
        return this.mainNet_;
    }
    getPlugin() {
        return this.plugin_;
    }
}
