var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { Json, Minute, Second } from "mvx-utils";
import { SignNonceStr } from "../Sign";
import { Token } from "../model/Token";
export var Code;
(function (Code) {
    Code[Code["Success"] = 200] = "Success";
    Code[Code["TokenExpire"] = 401] = "TokenExpire";
    Code[Code["ElseError"] = 500] = "ElseError";
})(Code || (Code = {}));
export class CodeError extends Error {
    constructor(err, code = Code.ElseError) {
        super();
        this.code = code;
        this.name = "CodeError";
        if (typeof err === 'string') {
            this.message = `${err}`;
        }
        else {
            this.message = `${err.message}`;
            this.stack = err.stack;
        }
    }
}
function newHttpOpt(...opts) {
    let opt = {
        header: new Map(),
        encoder: new class {
            encode(request) {
                throw new Error(`encoder not found, please call WithJsonBody、WithFormUrlEncodedBody、WithBody`);
            }
        },
        method: "GET",
        setToken: (opt, token) => { },
        cacheTTL: (code) => { return 0; },
        cacheAble: false,
        decoder: new class {
            decode(uri, resType, res, err) {
                const retStr = Object.prototype.toString.call(new resType()) == "[object String]";
                console.log("decoder.decode", "uri", uri, "res", res, "retStr", retStr);
                if (retStr) {
                    return [res, err];
                }
                if (err.code !== Code.Success) {
                    return [new resType(), err];
                }
                if (res === "") {
                    return [new resType(), new CodeError("response is empty")];
                }
                let [resT, err1] = new Json().fromJson(res, resType);
                if (err1) {
                    return [new resType(), new CodeError(err1)];
                }
                return [resT, null];
            }
        },
    };
    opts.forEach(f => {
        f(opt);
    });
    return opt;
}
export function WithHeader(header) {
    return (o) => {
        header.forEach((value, key) => {
            o.header.set(key, value);
        });
        return null;
    };
}
export function WithBearerToken() {
    return (o) => {
        o.setToken = (opt, token) => {
            o.header.set("Authorization", `Bearer ${token}`);
        };
        return null;
    };
}
export function WithBodyToken(tokenKey = "token") {
    return (o) => {
        o.setToken = (opt, token) => {
            opt.request[tokenKey] = token;
        };
        return null;
    };
}
export function WithoutToken() {
    return (o) => {
        o.setToken = (opt, token) => { };
        return null;
    };
}
export function WithReqId() {
    return (o) => {
        o.header.set("X-Req-Id", SignNonceStr());
        return null;
    };
}
export function WithBody(encoder) {
    return (o) => {
        o.encoder = encoder;
        return null;
    };
}
export function WithFormUrlEncodedBody() {
    return (o) => {
        o.header.set("X-Requested-With", "XMLHttpRequest");
        o.header.set('Content-Type', 'application/x-www-form-urlencoded');
        o.encoder = new class {
            encode(request) {
                const [json, err] = new Json().toJson(request);
                if (err) {
                    return ["", err];
                }
                const args = JSON.parse(json);
                const ret = Object.keys(args).map((key) => {
                    if (args[key] == null) {
                        return encodeURIComponent(key) + "=";
                    }
                    return encodeURIComponent(key) + "=" + encodeURIComponent(args[key]);
                }).join("&");
                return [ret, null];
            }
        };
        return null;
    };
}
export function WithJsonBody() {
    return (o) => {
        o.header.set("X-Requested-With", "XMLHttpRequest");
        o.header.set('Content-Type', 'application/json;charset=utf8');
        o.encoder = new class {
            encode(request) {
                return new Json().toJson(request);
            }
        };
        return null;
    };
}
export function WithHttpMethod(method) {
    return (o) => {
        o.method = method;
        return null;
    };
}
export function WithCache(cacheAble, cacheTTL = (code => code === Code.Success ? 1 * Minute : 5 * Second)) {
    return (o) => {
        o.cacheTTL = cacheTTL;
        o.cacheAble = cacheAble;
        return null;
    };
}
export function WithHttpDecoder(decoder) {
    return (o) => {
        o.decoder = decoder;
        return null;
    };
}
const cache = new Map();
const waiting = new Map();
export function SendNoToken(uri, request, resType, net, ...opts) {
    return __awaiter(this, void 0, void 0, function* () {
        // 兼容uri带有参数情况
        if (uri.lastIndexOf("?") >= 0) {
            const parts = uri.substring(uri.lastIndexOf("?") + 1).split("&");
            parts.forEach(r => {
                const args = r.split('=');
                if (args && args.length >= 2) {
                    request[args[0]] = args[1];
                }
            });
            uri = uri.substring(0, uri.lastIndexOf("?"));
        }
        const opt = newHttpOpt(...opts);
        opt.setToken({ header: opt.header, request: request }, yield net.getToken());
        const [body, err0] = opt.encoder.encode(request);
        if (err0) {
            return [new resType(), new CodeError(err0.message)];
        }
        const builder = net.getHttpBuilder();
        builder.setUri(uri).setMethod(opt.method).setHeaders(opt.header).setContent(body);
        const reqId = `${builder.method()}@${builder.getUrl()}/${builder.content()}`;
        if (opt.cacheAble && cache.has(reqId)) {
            console.log(`[${builder.method()}] ${builder.getUrl()} use cache req `, builder.content());
            const response = cache.get(reqId);
            const [res, err] = response.value;
            console.log(`[${builder.method()}]  ${builder.getUrl()} use cache res `, response);
            return opt.decoder.decode(uri, resType, res, err);
        }
        console.log(`[${builder.method()}] ${builder.getUrl()} send req `, builder.content());
        if (!waiting.has(reqId)) {
            waiting.set(reqId, []);
        }
        const promise = new Promise(resolve => {
            waiting.get(reqId).push({ resolve: resolve, opt: opt, resType: resType });
        });
        if (waiting.get(reqId).length > 1) {
            return promise;
        }
        const [res, err] = yield builder.build().send();
        console.log(`[${builder.method()}]  ${builder.getUrl()} send request `, [res, err]);
        if (opt.cacheAble) {
            const cacheTTL = opt.cacheTTL(err.code);
            cache.set(reqId, { value: [res, err], expireTime: new Date().getTime() + cacheTTL });
        }
        waiting.get(reqId).forEach(r => {
            const ret = r.opt.decoder.decode(uri, r.resType, res, err);
            r.resolve(ret);
            console.log(`[${builder.method()}]  ${builder.getUrl()} decode response `, ret);
        });
        waiting.set(reqId, []);
        return promise;
    });
}
export function Send(uri, request, resType, net, ...opts) {
    return __awaiter(this, void 0, void 0, function* () {
        if (net.is401()) {
            return [new resType(), new CodeError("has 401", Code.TokenExpire)];
        }
        let token = yield net.getToken();
        // token为空，需要处理401，并且不执行真正的网络请求。
        if (token === Token.empty) {
            yield net.process401();
            return [new resType(), new CodeError("token is empty", Code.TokenExpire)];
        }
        let [ret, err] = yield SendNoToken(uri, request, resType, net, ...opts);
        if (!err || err.code !== Code.TokenExpire) {
            // 取消401
            net.clear401();
            return [ret, err];
        }
        // 返回401时，如果请求的token与现在存储token不是一样的，说明有登录接口修改过，最新
        // 存储的token是否过期，无法判断，所以不能执行真正的401操作
        // 此处不能通过net层is401来判断，还没有开始处理401或者其他接口处理完401的情况下，is401都是false
        // 但是后一种情况却不能执行401；对token而言后一种情况token一定不一样，却可以分辨出两种情况
        if (token !== (yield net.getToken())) {
            return [new resType(), new CodeError("token is too old", Code.TokenExpire)];
        }
        yield net.process401();
        return [new resType(), new CodeError("token is expire", Code.TokenExpire)];
    });
}
