export class HttpBuilder {
    constructor() {
        this.baseUrl_ = "";
        this.uri_ = "";
        this.headers_ = new Map([]);
        this.content_ = "";
        this.method_ = "GET";
    }
    setContent(content) {
        this.content_ = content;
        return this;
    }
    content() {
        return this.content_;
    }
    setMethod(method) {
        this.method_ = method;
        return this;
    }
    method() {
        return this.method_;
    }
    setBaseUrl(url) {
        this.baseUrl_ = url;
        return this;
    }
    baseUrl() {
        return this.baseUrl_;
    }
    setUri(uri) {
        this.uri_ = uri;
        return this;
    }
    uri() {
        return this.uri_;
    }
    setHeaders(headers) {
        this.headers_ = headers;
        return this;
    }
    addHeader(key, val) {
        this.headers_.set(key, val);
        return this;
    }
    headers() {
        return this.headers_;
    }
    getUrl() {
        const trimHeadEndSeparator = (str) => {
            str = str.replace(/^\//, "");
            str = str.replace(/\/$/, "");
            return str;
        };
        let parts = [trimHeadEndSeparator(this.baseUrl())];
        parts.push(trimHeadEndSeparator(this.uri()));
        if (this.uri().search(/^(http|https):\/\//) >= 0) {
            parts = [this.uri()];
        }
        let url = parts.join("/");
        if (this.method() === "GET" && this.content().length) {
            url = `${url}?${this.content()}`;
        }
        return url;
    }
}
