import {Http} from "./Http";
import {CodeError} from "../api";

export type Method =
  | 'get' | 'GET'
  | 'delete' | 'DELETE'
  | 'head' | 'HEAD'
  | 'options' | 'OPTIONS'
  | 'post' | 'POST'
  | 'put' | 'PUT'
  | 'patch' | 'PATCH'
  | 'purge' | 'PURGE'
  | 'link' | 'LINK'
  | 'unlink' | 'UNLINK'

export abstract class HttpBuilder {
  public abstract build():Http;

  protected baseUrl_ = "";
  protected uri_ = "";
  protected headers_: Map<string, string> = new Map<string, string>([]);
  protected content_:string = "";
  protected method_:Method = "GET";

  public setContent(content:string): this {
    this.content_ = content;
    return this;
  }

  public content():string {
    return this.content_;
  }

  public setMethod(method:Method): this {
    this.method_ = method;
    return this;
  }

  public method():Method {
    return this.method_;
  }

  public setBaseUrl(url:string): this {
    this.baseUrl_ = url;
    return this;
  }

  public baseUrl():string {
    return this.baseUrl_;
  }

  public setUri(uri:string): this {
    this.uri_ = uri;
    return this;
  }

  public uri():string {
    return this.uri_;
  }

  public setHeaders(headers: Map<string, string>): this {
    this.headers_ = headers;
    return this;
  }

  public addHeader(key:string, val: string): this {
    this.headers_.set(key, val);
    return this;
  }

  public headers():Map<string, string> {
    return this.headers_
  }

  public getUrl():string {
    const trimHeadEndSeparator = (str:string): string => {
      str = str.replace(/^\//, "");
      str = str.replace(/\/$/, "");
      return str
    };

    let parts = [trimHeadEndSeparator(this.baseUrl())];
    parts.push(trimHeadEndSeparator(this.uri()));
    if (this.uri().search(/^(http|https):\/\//)>=0) {
      parts = [this.uri()];
    }

    let url = parts.join("/");
    if (this.method()==="GET" && this.content().length) {
      url = `${url}?${this.content()}`;
    }

    return url;
  }
}
