var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { CodeError, SendNoToken, WithJsonBody, WithoutToken } from "./api";
import { AppInvalidEvent, AppReValidEvent, Me } from "../model/Me";
export function SendLoginWithRes(uri, request, resType, net, resTokenParser, ...options) {
    return __awaiter(this, void 0, void 0, function* () {
        let meModel = yield net.getPlugin().mf().getModel(Me);
        // 如下的设置需要先清除token，再失效Me，最后再postEvent
        // 主网络才需要操作Me及发App事件
        yield net.clearToken();
        if (net.isMainNet()) {
            yield meModel.InValidate();
            yield net.getPlugin().nc().post(new AppInvalidEvent());
        }
        let [ret, err] = yield SendNoToken(uri, request, resType, net, WithJsonBody(), WithoutToken(), ...options);
        if (err) {
            return [ret, err];
        }
        // 如下的设置需要先使Me有效后(初始化库)，再设置token，最后再postEvent
        // 主网络才需要操作Me及发App事件
        const { uid, token } = resTokenParser(ret);
        if (!uid || !token) {
            return [ret, new CodeError(`uid or token is empty`)];
        }
        net.isMainNet() ? (yield meModel.ReValidateWithUid(uid)) : null;
        yield net.setToken(token);
        net.isMainNet() ? (yield net.getPlugin().nc().post(new AppReValidEvent())) : null;
        return [ret, null];
    });
}
class Response {
    constructor() {
        this.uid = null;
        this.token = null;
    }
}
export function SendLogin(uri, request, net, ...options) {
    return __awaiter(this, void 0, void 0, function* () {
        let [, err] = yield SendLoginWithRes(uri, request, Response, net, ret => {
            return { uid: ret.uid, token: ret.token };
        }, ...options);
        return err;
    });
}
