import {DefaultConstructor} from "mvx-utils";
import {Net} from "./Net";
import {CodeError, HttpOption, SendNoToken, WithJsonBody, WithoutToken} from "./api";
import {AppInvalidEvent, AppReValidEvent, Me} from "../model/Me";

export async function SendLoginWithRes<T extends object>(uri:string, request: object,
                                                         resType:DefaultConstructor<T>,
                                                         net:Net,
                                                         resTokenParser:(res:T) => {uid:string, token:string},
                                                         ...options:HttpOption[])
  : Promise<[T, CodeError|null]>{
  let meModel = await net.getPlugin().mf().getModel(Me);
  // 如下的设置需要先清除token，再失效Me，最后再postEvent
  // 主网络才需要操作Me及发App事件
  await net.clearToken();
  if (net.isMainNet()) {
    await meModel.InValidate();
    await net.getPlugin().nc().post(new AppInvalidEvent());
  }

  let [ret, err] = await SendNoToken(uri, request, resType, net, WithJsonBody(), WithoutToken(), ...options);
  if (err) {
    return [ret, err];
  }

  // 如下的设置需要先使Me有效后(初始化库)，再设置token，最后再postEvent
  // 主网络才需要操作Me及发App事件
  const {uid, token} = resTokenParser(ret);
  if (!uid || !token) {
    return [ret, new CodeError(`uid or token is empty`)]
  }

  net.isMainNet()?(await meModel.ReValidateWithUid(uid)):null;
  await net.setToken(token);
  net.isMainNet()?(await net.getPlugin().nc().post(new AppReValidEvent())):null;

  return [ret, null];
}

class Response {
  uid:string|null=null;
  token:string|null=null;
}

export async function SendLogin(uri:string, request: object, net:Net, ...options:HttpOption[]): Promise<CodeError|null> {
  let [, err] = await SendLoginWithRes(uri, request, Response, net, ret => {
    return {uid:ret.uid!, token:ret.token!}
  }, ...options);
  return err
}
